﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace Mod.libs
{
    public static class Utils
    {
        public static void Heal(LimbBehaviour limb, float factor, bool healBooleanConditions=false, bool timeBased = true)
        {
            if (timeBased)
            {
                factor *= Time.deltaTime;
            }

            GeneralHealing(limb, factor);
            HealBurn(limb, factor);
            HealAcidFlesh(limb, factor);
            HealRottenFlesh(limb, factor);

            if (limb.SkinMaterialHandler.damagePoints.Max((c) => { return c.z; }) > 0f)
            {
                HealWounds(limb, factor);
            }

            if (healBooleanConditions)
            {
                HealGraveConditions(limb);
            }
        }

        public static Component CopyComponent(Component original, Component destination)
        {
            System.Type type = original.GetType();
            System.Reflection.FieldInfo[] fields = type.GetFields();
            foreach (System.Reflection.FieldInfo field in fields)
            {
                field.SetValue(destination, field.GetValue(original));
            }
            return destination;
        }
        static float DecreaseClamp(float value, float amount, float target=0f)
        {
            return Math.Max(value - amount, target);
        }

        static float ApproachClamp(float value, float amount, float target)
        {
            if (target < value)
            {
                return DecreaseClamp(value, amount, target);
            }
            else
            {
                return Math.Min(value + amount, target);
            }
        }

        public static void HealGraveConditions(LimbBehaviour limb)
        {
            limb.HealBone();
            limb.CirculationBehaviour.HealBleeding();
            limb.CirculationBehaviour.IsPump = limb.CirculationBehaviour.WasInitiallyPumping;
        }

        public static void GeneralHealing(LimbBehaviour limb, float factor)
        {
            
            limb.Health = Math.Min(limb.InitialHealth, limb.Health + factor);
            limb.Numbness = DecreaseClamp(limb.Numbness, factor);

            limb.CirculationBehaviour.BloodFlow = Math.Min(5f, limb.CirculationBehaviour.BloodFlow + factor*5); ;
            limb.CirculationBehaviour.AddLiquid(limb.GetOriginalBloodType(), factor*5);
            limb.BruiseCount = (ushort) DecreaseClamp(limb.BruiseCount, factor);
            limb.CirculationBehaviour.GunshotWoundCount = (ushort) DecreaseClamp(limb.CirculationBehaviour.GunshotWoundCount, factor);
            limb.CirculationBehaviour.StabWoundCount = (ushort) DecreaseClamp(limb.CirculationBehaviour.StabWoundCount, factor);
            if (limb.RoughClassification == LimbBehaviour.BodyPart.Head)
            {
                limb.Person.Consciousness = Math.Min(1f, limb.Person.Consciousness + factor);
                limb.Person.ShockLevel = DecreaseClamp(limb.Person.ShockLevel, factor);
                limb.Person.PainLevel = DecreaseClamp(limb.Person.PainLevel, factor);
                limb.Person.OxygenLevel = Math.Min(1f, limb.Person.OxygenLevel + factor);
                limb.Person.AdrenalineLevel = Math.Min(1f, limb.Person.AdrenalineLevel + factor);
            }
        }

        public static void HealBurn(LimbBehaviour limb, float factor)
        {
            limb.PhysicalBehaviour.Temperature = ApproachClamp(limb.PhysicalBehaviour.Temperature, factor, limb.BodyTemperature);
            limb.PhysicalBehaviour.BurnProgress = DecreaseClamp(limb.PhysicalBehaviour.BurnProgress, factor*0.16f); //*0.16
        }

        public static void HealAcidFlesh(LimbBehaviour limb, float factor)
        {
            limb.SkinMaterialHandler.AcidProgress = DecreaseClamp(limb.SkinMaterialHandler.AcidProgress, factor*0.8f);
        }

        public static void HealRottenFlesh(LimbBehaviour limb, float factor)
        {
            limb.SkinMaterialHandler.RottenProgress = DecreaseClamp(limb.SkinMaterialHandler.RottenProgress, factor * 0.8f);
        }

        public static void HealWounds(LimbBehaviour limb, float factor)
        {
            Vector4[] damagePoints = limb.SkinMaterialHandler.damagePoints;
            for (int i = 0; i < limb.SkinMaterialHandler.damagePoints.Length; i++)
            {
                damagePoints[i].z = DecreaseClamp(damagePoints[i].z, factor*0.5f);
            }
            limb.SkinMaterialHandler.Sync();

            /*while (limb.SkinMaterialHandler.damagePoints.Max((c) => { return c.z; }) > 0f)
            {
                
            }*/
        }
    }
}
