﻿using System;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;
using UnityEngine.Animations;

namespace Mod
{
    public class LiquidGun : MonoBehaviour
    {
        public const string name = "​<b></b>" + "Liquid Gun";
        public static void Load()
        {
            GorseProjectileBehaviour projectileBehav = Resources.FindObjectsOfTypeAll<GorseProjectileBehaviour>()[0];
            HealthGunBehaviour healthGun = Resources.FindObjectsOfTypeAll<HealthGunBehaviour>()[0];

            GameObject gorseProjectile = projectileBehav.gameObject;
            //GameObject disperseEffect = (GameObject)Resources.InstanceIDToObject(16356);
            Sprite reconstructorThingySprite = healthGun.GetComponent<SpriteRenderer>().sprite;

            var reconstructor = ModAPI.FindSpawnable("Reconstructor").Prefab;
            BoxCollider2D[] reconstructorColliderShape = reconstructor.GetComponents<BoxCollider2D>();

            Modification modification = new Modification()
            {
                OriginalItem = ModAPI.FindSpawnable("Crossbow"),

                NameOverride = name,
                DescriptionOverride = "Gun that launches the liquid contained within itself.",

                CategoryOverride = ModAPI.FindCategory("Firearms"),
                AfterSpawn = (Instance) =>
                {
                    Instance.GetComponent<SpriteRenderer>().sprite = reconstructorThingySprite;
                    foreach (Collider2D col in Instance.GetComponents<Collider2D>())
                    {
                        Destroy(col);
                    }

                    foreach (BoxCollider2D boxCollider in reconstructorColliderShape)
                    {
                        var collider = Instance.AddComponent<BoxCollider2D>();
                        collider.offset = boxCollider.offset;
                        collider.size = boxCollider.size;
                    }
                    Instance.GetComponent<PhysicalBehaviour>().ResetColliderArray();
                    


                    LiquidGun liquidGun = Instance.GetOrAddComponent<LiquidGun>();
                    ProjectileLauncherBehaviour projectileLauncher = Instance.GetOrAddComponent<ProjectileLauncherBehaviour>();

                    projectileLauncher.projectileAsset = null;
                    projectileLauncher.projectilePrefab = gorseProjectile;

                    projectileLauncher.OnLaunch += (sender, projectile) =>
                    {
                        if (liquidGun.flask.TotalLiquidAmount <= 0f)
                        {
                            Destroy(projectile);
                            return;
                        }

                        var launcher = (ProjectileLauncherBehaviour)sender;

                        Quaternion rotation = (launcher.transform.lossyScale.x < 0f) ? (Quaternion.Euler(0f, 0f, 180f) * launcher.transform.rotation) : launcher.transform.rotation;
                        projectile.GetComponent<Rigidbody2D>().AddForce(rotation * Vector2.right, ForceMode2D.Impulse);

                        foreach (Collider2D collider in launcher.transform.root.GetComponentsInChildren<Collider2D>())
                        {
                            IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(projectile.GetComponent<Collider2D>(), collider);
                        }

                        Destroy(projectile.GetComponent<GorseProjectileBehaviour>());
                        var projBehaviour = projectile.AddComponent<CustomLiquidProjectileBehaviour>();
                        //projBehaviour.disperseEffect = disperseEffect;

                        Color color = liquidGun.flask.GetComputedColor();

                        ParticleSystem particles = projectile.GetComponent<ParticleSystem>();
                        var main = particles.main;
                        main.startColor = color;

                        var renderer = projectile.GetComponentInChildren<SpriteRenderer>();
                        renderer.color = color;
                        

                        projBehaviour.totalLiquidAmount = liquidGun.flask.TotalLiquidAmount;
                        projBehaviour.liquidDistribution = CloneLiquidDistribution(liquidGun.flask.LiquidDistribution);
                        projBehaviour.color = color;
                        liquidGun.flask.Drain(0.1f);
                    };
                }
            };




            ModAPI.Register(modification);
            Mod.nailgun.Nail(name, modification.AfterSpawn);
        }

        public static Dictionary<Liquid, BloodContainer.RefFloat> CloneLiquidDistribution(Dictionary<Liquid, BloodContainer.RefFloat> original)
        {
            var ret = new Dictionary<Liquid, BloodContainer.RefFloat>(original.Count, original.Comparer);
            foreach (KeyValuePair<Liquid, BloodContainer.RefFloat> entry in original)
            {
                ret.Add(entry.Key, new BloodContainer.RefFloat(entry.Value.Raw));
            }
            return ret;
        }

        public FlaskBehaviour flask;
        public GameObject flaskObject;

        void Awake()
        {
            flaskObject = Instantiate(ModAPI.FindSpawnable("Flask").Prefab, gameObject.transform);
            flaskObject.GetComponent<SpriteRenderer>().sortingOrder = GetComponent<SpriteRenderer>().sortingOrder - 1;

            flaskObject.transform.position = transform.TransformPoint(new Vector2(ModAPI.PixelSize * 3f, ModAPI.PixelSize * 10f));
            flaskObject.transform.Rotate(new Vector3(180f, 0, 0));

            var constraint = flaskObject.AddComponent<FixedJoint2D>();
            constraint.connectedBody = GetComponent<Rigidbody2D>();
            constraint.autoConfigureConnectedAnchor = true;

            
            
            var phys = flaskObject.GetComponent<PhysicalBehaviour>();
            phys.Deletable = false;

            foreach (Collider2D collider in transform.root.GetComponentsInChildren<Collider2D>())
            {
                IgnoreCollisionStackController.IgnoreCollisionSubstituteMethod(phys.colliders[0], collider);
            }

                

            flask = flaskObject.GetComponent<FlaskBehaviour>();
        }

        public class CustomLiquidProjectileBehaviour : MonoBehaviour
        {
            public float totalLiquidAmount;
            public Dictionary<Liquid, BloodContainer.RefFloat> liquidDistribution;
            public Color color;
            public GameObject disperseEffect;

            void OnCollisionEnter2D(Collision2D collision)
            {
                LimbBehaviour limbBehaviour;
                if (collision.transform && collision.transform.TryGetComponent(out limbBehaviour))
                {
                    limbBehaviour.Wince(0.05f);
                    CopyTo(1f, limbBehaviour.CirculationBehaviour);
                }

                //var fx = Instantiate(disperseEffect, transform.position, Quaternion.identity);
                // Make custom blood explosion later since gorse one is weird

                Destroy(gameObject);
            }

            void CopyTo(float amount, BloodContainer target)
            {
                foreach (KeyValuePair<Liquid, BloodContainer.RefFloat> keyValuePair in liquidDistribution)
                {
                    Liquid key = keyValuePair.Key;
                    float num = keyValuePair.Value.Raw / totalLiquidAmount;
                    target.AddLiquid(key, amount * num);
                }
            }
        }

        
    }
}

//Author: Polars Bear
