﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using System.Reflection.Emit;
using TMPro;
using UnityEngine;
using UnityEngine.Events;

namespace Mod
{
    public class CustomSyringe : SyringeBehaviour
    {
        public const string name = "​<b></b>" + "Custom Syringe";
        public static Liquid spawnLiquid;
        public static void Load()
        {
            spawnLiquid = new CustomSyringeSerum();
            ModAPI.RegisterLiquid(CustomSyringeSerum.ID, spawnLiquid);

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = name,
                    DescriptionOverride = "Syringe that does (almost) whatever you want it to! Right click it to set its color and different properties. The effects are global, however, so every syringe, container and person with the serum will be affected.",

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<CustomSyringe>(); }
                }
            );
            Mod.nailgun.SyringeNail(name, typeof(CustomSyringe));
        }
        void Start()
        {
            AddCustomButtons();
            base.Start();
        }
        

        /*protected override void FixedUpdate()
        {
            if (!Finite && GetLiquidID() != null)
            {
                AddLiquid(spawnLiquid, Limits.y - GetAmount(spawnLiquid));
            }
            foreach (BloodContainer other in pushTargets)
            {
                DecompiledClass stuff;
                stuff.other = other;
                if (stuff.other)
                {
                    AveragePressure(Time.fixedDeltaTime, this, stuff.other);
                    if (!Finite || MeasuredPressure > stuff.other.MeasuredPressure)
                    {
                        PushLiquid(ref stuff);
                    }

                    else if (Finite)
                    {
                        PullLiquid(ref stuff);
                    }
                }
            }
        }

        void PushLiquid(ref DecompiledClass A_1)
        {
            if (TotalLiquidAmount <= LowerLimit || A_1.other.TotalLiquidAmount >= A_1.other.UpperLimit)
            {
                return;
            }
            if (GetLiquidID() != null)

            {
                CirculationBehaviour circulationBehaviour = A_1.other as CirculationBehaviour;
                if (circulationBehaviour == null)
                {
                    TransferTo(TransferRate, A_1.other, true);
                    return;
                }
                if (circulationBehaviour.GetAmount(spawnLiquid) < 0.5f)
                {
                    TransferTo(TransferRate, A_1.other, true);
                    return;
                }
            }
            else

            {
                TransferTo(TransferRate, A_1.other, true);
            }
        }

        void PullLiquid(ref DecompiledClass A_1)
        {
            if (A_1.other.TotalLiquidAmount <= A_1.other.LowerLimit || TotalLiquidAmount >= UpperLimit)
            {
                return;
            }
            A_1.other.TransferTo(TransferRate, this, true);
        }

        public static void AveragePressure(float rate, BloodContainer from, BloodContainer to)
        {
            if (!from || !to)
            {
                return;
            }
            if (rate < 0f)
            {
                throw new Exception("Rate can't be negative");
            }
            float measuredPressure = from.MeasuredPressure;
            float measuredPressure2 = to.MeasuredPressure;
            float b = 0.5f * (measuredPressure + measuredPressure2);
            if (from.AllowPressureTransfer && to.AllowPressureTransfer)
            {
                from.MeasuredPressure = Mathf.Lerp(measuredPressure, b, rate);
                to.MeasuredPressure = Mathf.Lerp(measuredPressure2, b, rate);
            }
        }
        public struct DecompiledClass
        {
            public BloodContainer other;
        }*/

        bool ContainsEffect(int coro)
        {
            return CustomSyringeSerum.EffectCoroutines.Contains(CustomSyringeEffectCoroutines.Coroutines[coro]);
        }

        void AddEffect(int coro)
        {
            CustomSyringeSerum.EffectCoroutines.Add(CustomSyringeEffectCoroutines.Coroutines[coro]);
        }

        void RemoveEffect(int coro)
        {
            CustomSyringeSerum.EffectCoroutines.Remove(CustomSyringeEffectCoroutines.Coroutines[coro]);
        }

        struct ButtonData
        {
            public int id;
            public string name;
            public string desc;

            public ButtonData(string name, string desc)
            {
                this.id = -1;
                this.name = name;
                this.desc = desc;
            }
        }

        ButtonData[] buttons = new ButtonData[]
        {
            new ButtonData("Damaging", "Toggle syringe damaging"),
            new ButtonData("Glowing", "Toggle syringe glowing")
        };

        void ForceRecalculateLiquidColors()
        {
            foreach (BloodContainer container in CustomSyringeSerum.containers)
            {
                container.ForceCalculateComputedColor(container.GetComputedColor());
            }
        }

        void AddCustomButtons()
        {
            List<ContextMenuButton> contextButtons = GetComponent<PhysicalBehaviour>().ContextMenuOptions.Buttons;
            for (int i = 0; i < buttons.Length; i++)
            {
                ButtonData data = buttons[i];
                data.id = i;
                ContextMenuButton damagingOn = new ContextMenuButton(() => { return !ContainsEffect(data.id); }, $"set{data.name}On", $"{data.name} (OFF)", data.desc, new UnityAction[] { () => { AddEffect(data.id); } });
                ContextMenuButton damagingOff = new ContextMenuButton(() => { return ContainsEffect(data.id); }, $"set{data.name}Off", $"{data.name} (ON)", data.desc, new UnityAction[] { () => { RemoveEffect(data.id); } });

                contextButtons.Add(damagingOn);
                contextButtons.Add(damagingOff);
            }

            ContextMenuButton hardness = new ContextMenuButton("setHardness", "Set Hardness", "Set the hardness that will be applied to the user", new UnityAction[] {delegate
            {
                DialogBox dialog = null;
                    dialog = DialogBoxManager.TextEntry("Hardness", "0-100", new DialogButton[] {
                        new DialogButton("Set", true, new UnityAction[] { delegate {
                            float num;
                            if (float.TryParse(dialog.InputField.text, out num))
                            {
                                SetHardness(num/100f);
                            }
                        }})});
            }});

            contextButtons.Add(hardness);

            string[] colors = new string[] { "Red", "Green", "Blue" };
            for (int i = 0; i < 3; i++)
            {
                int index = i;
                ContextMenuButton set = new ContextMenuButton($"set{colors[index]}", $"Set {colors[index]}", $"Set {colors[index].ToUpper()} value of color", new UnityAction[] { delegate {
                    DialogBox dialog = null;
                    dialog = DialogBoxManager.TextEntry(colors[index].ToUpper(), "0-255", new DialogButton[] {
                        new DialogButton("Set", true, new UnityAction[] { delegate {
                            float num;
                            if (float.TryParse(dialog.InputField.text, out num))
                            {
                                spawnLiquid.Color[index] = num/255f;
                                ForceRecalculateLiquidColors();
                            }
                        }})});
                }});
                contextButtons.Add(set);
            }

            
        }
        static void SetHardness(float hardness)
        {
            CustomSyringeSerum.customPhysicalProps.Softness = 0.35f - (hardness * 0.35f);
            CustomSyringeSerum.customPhysicalProps.BulletSpeedAbsorptionPower = hardness * 3f;
            CustomSyringeSerum.customPhysicalProps.ImpactIntensityMutliplier = 0.25f - (hardness * 0.25f);
        }

        public override string GetLiquidID() => CustomSyringeSerum.ID;
        public class CustomSyringeSerum : Liquid
        {
            public static List<Func<Liquid, LimbBehaviour, IEnumerator>> EffectCoroutines = new List<Func<Liquid, LimbBehaviour, IEnumerator>>();
            public static PhysicalProperties customPhysicalProps;
            public static List<BloodContainer> containers = new List<BloodContainer>();

            public const string ID = "​<b></b>" + "CUSTOM SERUM";
            public CustomSyringeSerum()
            {
                Color = new Color(0f, 0f, 0f);
                foreach(PhysicalProperties props in Resources.LoadAll<PhysicalProperties>("PhysicalProperties")) {
                    Debug.Log(props.name);
                }
                customPhysicalProps = ModAPI.FindPhysicalProperties("Human").ShallowClone();
            }
            public override void OnEnterLimb(LimbBehaviour limb)
            {
                foreach (Func<Liquid, LimbBehaviour, IEnumerator> effect in EffectCoroutines)
                {
                    limb.StartCoroutine(effect.Invoke(this, limb));
                }
                limb.PhysicalBehaviour.Properties = customPhysicalProps;
            }
            public override void OnExitContainer(BloodContainer container) { containers.Remove(container); }
            public override void OnUpdate(BloodContainer container) { }
            public override void OnEnterContainer(BloodContainer container) { containers.Add(container); }

            public IEnumerator DisappearLiquid(LimbBehaviour limb)
            {
                CirculationBehaviour circulation = limb.CirculationBehaviour;
                float quantity = circulation.GetAmount(this);
                while (quantity > 1E-45f)
                {
                    circulation.RemoveLiquid(this, Math.Max(quantity, 0.001f * Time.deltaTime));
                    yield return new WaitForEndOfFrame();
                    quantity = circulation.GetAmount(this);
                }
            }
        }

        public static class CustomSyringeEffectCoroutines
        {
            public static Func<Liquid, LimbBehaviour, IEnumerator>[] Coroutines = new Func<Liquid, LimbBehaviour, IEnumerator>[]
            {
                DamageCoroutine,
                GlowCoroutine
            };

            public static IEnumerator DamageCoroutine(Liquid liquid, LimbBehaviour limb)
            {
                CirculationBehaviour circulation = limb.CirculationBehaviour;
                float quantity = circulation.GetAmount(liquid);
                float rngTimer = 0f;
                while (quantity > 1E-45f)
                {
                    float rng = UnityEngine.Random.Range(0, 500)*Time.deltaTime;
                    rngTimer += rng;
                    if (rngTimer > 20)
                    {
                        Vector2 limits = limb.transform.localScale;
                        float x = UnityEngine.Random.Range(0, limits.x);
                        float y = UnityEngine.Random.Range(0, limits.y);
                        Vector2 point = new Vector2(x - limits.x * 0.5f, y - limits.y * 0.5f);
                        limb.Shot(new Shot(Vector2.down, limb.transform.TransformPoint(point), quantity * 0.2f));
                        rngTimer -= 20;
                    }
                    yield return new WaitForEndOfFrame();
                    quantity = circulation.GetAmount(liquid);
                }
            }

            public static IEnumerator GlowCoroutine(Liquid liquid, LimbBehaviour limb)
            {
                CirculationBehaviour circulation = limb.CirculationBehaviour;
                float quantity = circulation.GetAmount(liquid);
                LightSprite light = null;
                foreach (LightSprite lightSprite in limb.transform.GetComponentsInChildren<LightSprite>())
                {
                    if (lightSprite.gameObject.name == "CustomSerumLight")
                    {
                        light = lightSprite;
                    }
                }
                if (light == null)
                {
                    light = ModAPI.CreateLight(limb.transform, liquid.Color, quantity * 5f);
                    light.gameObject.name = "CustomSerumLight";
                }
                while (quantity > 1E-45f)
                {
                    light.enabled = true;
                    light.Radius = quantity * 5f;
                    light.Color = liquid.Color;
                    yield return new WaitForEndOfFrame();
                    quantity = circulation.GetAmount(liquid);
                }
            }
        }
    }
}

//Author: Polars Bear
